﻿using DotNetCore.CAP;
using Hotline.Application.FlowEngine;
using Hotline.FlowEngine.Notifications;
using Hotline.FlowEngine.WorkflowModules;
using Hotline.KnowledgeBase;
using Hotline.Orders;
using Hotline.Repository.SqlSugar.CallCenter;
using Hotline.Repository.SqlSugar.Orders;
using Hotline.Settings;
using Hotline.Share.Dtos.FlowEngine.Workflow;
using Hotline.Share.Dtos.Order;
using Hotline.Share.Enums.Order;
using Hotline.Share.Mq;
using MapsterMapper;
using MediatR;
using Microsoft.Extensions.Logging;
using Newtonsoft.Json;
using XF.Domain.Entities;
using XF.Domain.Exceptions;
using XF.Domain.Repository;

namespace Hotline.Application.Handlers.FlowEngine;

public class NextStepHandler : INotificationHandler<NextStepNotify>
{
    private readonly IOrderDomainService _orderDomainService;
    private readonly IKnowledgeDomainService _knowledgeDomainService;
    private readonly IOrderRepository _orderRepository;
    private readonly ICapPublisher _capPublisher;
    private readonly IMapper _mapper;
    private readonly ILogger<NextStepHandler> _logger;
    private readonly IRepository<OrderScreen> _orderScreenRepository;

    public NextStepHandler(
        IOrderDomainService orderDomainService,
        IKnowledgeDomainService knowledgeDomainService,
        IOrderRepository orderRepository,
        ICapPublisher capPublisher,
        IMapper mapper,
        ILogger<NextStepHandler> logger,
        IRepository<OrderScreen> orderScreenRepository)
    {
        _orderDomainService = orderDomainService;
        _knowledgeDomainService = knowledgeDomainService;
        _orderRepository = orderRepository;
        _capPublisher = capPublisher;
        _mapper = mapper;
        _logger = logger;
        _orderScreenRepository = orderScreenRepository;
    }

    /// <summary>Handles a notification</summary>
    /// <param name="notification">The notification</param>
    /// <param name="cancellationToken">Cancellation token</param>
    public async Task Handle(NextStepNotify notification, CancellationToken cancellationToken)
    {
        _logger.LogInformation($"收到{nameof(NextStepNotify)}, notification: {JsonConvert.SerializeObject(notification)}");
        var workflow = notification.Workflow;
        var data = notification.Dto;

        DefinitionTag? currentTag = null;
        if (!string.IsNullOrEmpty(notification.Trace.Tag))
            currentTag = System.Text.Json.JsonSerializer.Deserialize<DefinitionTag>(notification.Trace.Tag);
        DefinitionTag? nextTag = null;
        if (!string.IsNullOrEmpty(notification.nextStepDefine.Tag))
            nextTag = System.Text.Json.JsonSerializer.Deserialize<DefinitionTag>(notification.nextStepDefine.Tag);


        switch (workflow.ModuleCode)
        {
            case WorkflowModuleConsts.OrderHandle:
                var order = await _orderDomainService.GetOrderAsync(workflow.ExternalId, cancellationToken);
                order.CheckIfFiled();
                _mapper.Map(workflow, order);
                await _orderRepository.UpdateAsync(order, cancellationToken);

                var orderDto = _mapper.Map<OrderDto>(order);
                if (notification.IsCenterToOrg)
                    await _capPublisher.PublishAsync(EventNames.HotlineOrderExpiredTimeUpdate, orderDto, cancellationToken: cancellationToken);

                if (notification.HandlerOrgCode.CheckIfOrgLevelIs(1))
                    await _capPublisher.PublishAsync(EventNames.HotlineOrderLevelOneOrgHandled, new OrderFlowDto
                    {
                        Order = orderDto,
                        WorkflowTrace = _mapper.Map<WorkflowTraceDto>(notification.Trace)
                    }, cancellationToken: cancellationToken);

                if (currentTag is not null)
                {
                    if (currentTag.Type == TagDefaults.TagType.Org)
                    {
                        switch (currentTag.Value)
                        {
                            case TagDefaults.TagValue.Province:

                                break;
                        }
                    }
                }

                break;
            case WorkflowModuleConsts.KnowledgeAdd:
            case WorkflowModuleConsts.KnowledgeUpdate:
            case WorkflowModuleConsts.KnowledgeDelete:
            case WorkflowModuleConsts.OrderScreen:
                var screen = await _orderScreenRepository.Queryable().Includes(x=>x.Order).Where(x=>x.Id == workflow.ExternalId).FirstAsync(cancellationToken);
                if (screen != null)
                {
                    screen.Status = EScreenStatus.Approval;
                    await _orderScreenRepository.UpdateAsync(screen, cancellationToken);
                }
                if (nextTag is not null)
                {
	                if (nextTag.Type == TagDefaults.TagType.Org)
	                {
		                switch (nextTag.Value)
		                {
			                case TagDefaults.TagValue.Province:
                                if (screen != null)
                                {
                                    var screenDto = _mapper.Map<OrderScreenListDto>(screen);
                                    if (screen.Order != null && screen.Order.Source == ESource.ProvinceStraight)
                                    {
                                        var screenOrderDto = _mapper.Map<OrderDto>(screen.Order);
                                        //推省上
                                        _capPublisher.Publish(EventNames.HotlineOrderScreenApply, new PublishScreenDto()
                                        {
                                            Order = screenOrderDto,
                                            Screen = screenDto,
                                            ClientGuid = ""
                                        });
                                    }
                                }
                                break;
		                }
	                }
                }
				break;
            case WorkflowModuleConsts.OrderDelay:
                if (nextTag is not null)
                {
                    if (nextTag.Type == TagDefaults.TagType.Org)
                    {
                        switch (nextTag.Value)
                        {
                            case TagDefaults.TagValue.Province:
                                //发起省延期审批
                                break;
                        }
                    }
                }
                break;
        }
    }
}